/**
 * An EventDispatcher can be used to dispatch and listen to events.
 **/
class EventDispatcher
{
	constructor()
	{
		this.general_event_listeners = [];
		this.event_listeners = {};
	}
	
	event_listeners = {};
	general_event_listeners = [];
}

/**
 * Adds a listener for a specific event type.
 * @param event_type int32 the type of event to listen to
 * @param listener function(event) the callback function
 **/
function EventDispatcher::AddEventListener(event_type, listener)
{
	if (!(event_type in this.event_listeners))
		this.event_listeners[event_type] <- [];
	this.event_listeners[event_type].push(listener);
}

function EventDispatcher::AddGeneralEventListener(listener)
{
	this.general_event_listeners.push(listener);
}

function EventDispatcher::DispatchEvent(event)
{
	foreach (i, listener in general_event_listeners)
		listener(event);
		
	if (!(event.event_type in this.event_listeners)) return;
	
	foreach (i, listener in this.event_listeners[event.event_type])
		listener(event);
}