require("EventDispatcher.nut");
require("SignObserverEvent.nut");

class SignObserver extends EventDispatcher
{
	constructor()
	{
		::EventDispatcher.constructor();
		signs = {};
		Check();
	}
	
	signs = null;
}

/* private */ function SignObserver::ActivateCompany(company_id)
{
	if (!(company_id in this.signs))
		this.signs[company_id] <- {};
}

function SignObserver::Check()
{
	for (local company_id = GSCompany.COMPANY_FIRST; company_id <= GSCompany.COMPANY_LAST; company_id++)
		this.CheckCompanySigns(company_id);
}

function SignObserver::CheckCompanySigns(company_id)
{
	local company_id = GSCompany.ResolveCompanyID(company_id);
	if (company_id == GSCompany.COMPANY_INVALID)
		return;
		
	this.ActivateCompany(company_id);

	local sign_list = this.GetCompanySigns(company_id);
	local our_sign_list = this.signs[company_id];

	foreach (sign_id, _ in sign_list)
	{
		local text = this.GetSignName(company_id, sign_id);
		if (!(sign_id in our_sign_list))
		{
			if (text == "Sign")
				continue;
			this.DispatchEvent(SignObserverEvent(SignObserverEvent.SOE_ADDED, company_id, sign_id, this.GetSignTile(company_id, sign_id), text));
			our_sign_list[sign_id] <- text;
		}
		else if (text != our_sign_list[sign_id])
		{
			this.DispatchEvent(SignObserverEvent(SignObserverEvent.SOE_CHANGED, company_id, sign_id, this.GetSignTile(company_id, sign_id), text));
			our_sign_list[sign_id] = text;
		}
	}
	
	if (sign_list.Count() < our_sign_list.len())
	{
		foreach (sign_id, text in our_sign_list)
		{
			if (!sign_list.HasItem(sign_id))
			{
				this.DispatchEvent(SignObserverEvent(SignObserverEvent.SOE_REMOVED, company_id, sign_id, this.GetSignTile(company_id, sign_id), text));
				delete our_sign_list[sign_id];
			}
		}
	}
}

function SignObserver::GetCompanySigns(company_id)
{
	local company = GSCompanyMode(company_id);
	return GSSignList();
}

function SignObserver::GetSignName(company_id, sign_id)
{
	local company = GSCompanyMode(company_id);
	return GSSign.GetName(sign_id);
}

function SignObserver::GetSignTile(company_id, sign_id)
{
	local company = GSCompanyMode(company_id);
	return GSSign.GetLocation(sign_id);
}