class WhackATree
{
	constructor(company_id, tile_id, sign_id)
	{
		this.company_id = company_id;
		this.tile_id = tile_id;
		this.sign_id = sign_id;
		this.tile_x = GSMap.GetTileX(tile_id);
		this.tile_y = GSMap.GetTileY(tile_id);
		is_running = false;
	}
	
	company_id = null;
	tile_id = null;
	sign_id = null;
	tile_x = null;
	tile_y = null;
	is_running = null;
	tree_list = null;
	score = null;
	
	score_sign_id = null;
	time_sign_id = null;
	start_date = null;
	current_date = null;
}

function WhackATree::Start()
{
	if (this.is_running) return;
	
	// Initialize some values
	tree_list = GSTileList();
	score = 0;
	
	// Enter company mode
	local company = GSCompanyMode(this.company_id);
	
	// Demolish board
	for (local y = -2; y < 3; ++y)
	{
		for (local x = -2; x < 3; ++x)
		{
			GSTile.DemolishTile(GSMap.GetTileIndex(this.tile_x + x, this.tile_y + y));
		}
	}
	
	// Retrieve start date
	this.start_date = GSDate.GetCurrentDate();
	this.current_date = this.start_date;
	
	// Remove starting sign and place game signs
	GSSign.RemoveSign(this.sign_id);
	this.time_sign_id = GSSign.BuildSign(GSMap.GetTileIndex(this.tile_x - 3, this.tile_y + 3), "Days left: " + GSController.GetSetting("game_length"));
	this.score_sign_id = GSSign.BuildSign(GSMap.GetTileIndex(this.tile_x + 3, this.tile_y - 3), "Score: ");
	
	// Set to running
	this.is_running = true;
}

function WhackATree::Finished()
{
	return !this.is_running;
}

function WhackATree::Stop()
{
	// In company mode...
	local company = GSCompanyMode(this.company_id);
	// we print the final score...
	GSSign.BuildSign(this.tile_id, "Final score: "+ this.score);
	// and remove the other signs...
	GSSign.RemoveSign(this.time_sign_id);
	GSSign.RemoveSign(this.score_sign_id);
	
	// which concludes the game.
	this.is_running = false;
}

function WhackATree::Step()
{
	local company = GSCompanyMode(this.company_id);
	
	// Iterate over our tree list and see if a tree was removed.
	foreach (tile_index, _ in this.tree_list)
	{
		if (!GSTile.HasTreeOnTile(tile_index))
		{
			this.tree_list.RemoveTile(tile_index);
			
			// We only score if the game was not paused.
			if (!GSGame.IsPaused())
				this.score++;
		}
	}
	
	// No spawns while game is paused.
	if (GSGame.IsPaused())
		return;
		
	// Update score sign.
	GSSign.SetName(this.score_sign_id, "Score: " + this.score);
	
	// Check if we want to spawn a tree.
	if (GSBase.Chance(GSController.GetSetting("tree_spawn_chance"), 100))
	{
		// Determine tree location
		local x = GSBase.RandRange(5) - 2;
		local y = GSBase.RandRange(5) - 2;
		local tile_index = GSMap.GetTileIndex(this.tile_x + x, this.tile_y + y);
		
		// We do nothing if a tree already exists on the target tile.
		if (!GSTile.HasTreeOnTile(tile_index))
		{
			GSTile.PlantTree(tile_index);
			this.tree_list.AddTile(tile_index);
		}
	}
	
	// Update date.
	local new_date = GSDate.GetCurrentDate();
	if (new_date != this.current_date)
	{
		this.current_date = new_date;
		// Check if we're done.
		if (this.current_date - this.start_date >= GSController.GetSetting("game_length"))
			this.Stop();
		// Update time sign.
		GSSign.SetName(this.time_sign_id, "Days left: " + (GSController.GetSetting("game_length") - (this.current_date - this.start_date)));
	}
}